#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2023 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from collections import namedtuple

import apparmor.severity as severity

from apparmor.common import AppArmorBug, AppArmorException
from apparmor.rule.all import AllRule, AllRuleset
from apparmor.translations import init_translation
from common_test import AATest, setup_all_loops

_ = init_translation()

exp = namedtuple(
    'exp', ('audit', 'allow_keyword', 'deny', 'comment',
            # no localvars
            ))

# --- tests for single AllRule --- #


class AllTest(AATest):
    def _compare_obj(self, obj, expected):
        self.assertEqual(expected.allow_keyword, obj.allow_keyword)
        self.assertEqual(expected.audit, obj.audit)
        self.assertEqual(expected.deny, obj.deny)
        self.assertEqual(expected.comment, obj.comment)


class AllTestParse(AllTest):
    tests = (
        # rawrule                       audit  allow  deny   comment
        ('all,',                    exp(False, False, False, '')),
        ('deny all, # comment',     exp(False, False, True,  ' # comment')),
        ('audit allow all,',        exp(True,  True,  False, '')),
        ('audit allow all,',        exp(True,  True,  False, '')),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(AllRule.match(rawrule))
        obj = AllRule.create_instance(rawrule)
        self.assertEqual(rawrule.strip(), obj.raw_rule)
        self._compare_obj(obj, expected)


class AllTestParseInvalid(AllTest):
    tests = (
        ('all -> ,',        AppArmorException),
        ('owner all,',      AppArmorException),
        ('all foo ,',       AppArmorException),
    )

    def _run_test(self, rawrule, expected):
        self.assertFalse(AllRule.match(rawrule))
        with self.assertRaises(expected):
            AllRule.create_instance(rawrule)


# we won't ever support converting a log event to an 'all,' rule
# class AllTestParseFromLog(AllTest):


class AllFromInit(AllTest):
    tests = (
        # AllRule object            audit  allow  deny   comment
        (AllRule(deny=True),    exp(False, False, True,  '')),
        (AllRule(),             exp(False, False, False, '')),
    )

    def _run_test(self, obj, expected):
        self._compare_obj(obj, expected)


# no localvars -> no way to hand over invalid values, or to miss a required parameter
# class InvalidAllInit(AATest):


class InvalidAllTest(AATest):
    def _check_invalid_rawrule(self, rawrule):
        obj = None
        self.assertFalse(AllRule.match(rawrule))
        with self.assertRaises(AppArmorException):
            obj = AllRule.create_instance(rawrule)

        self.assertIsNone(obj, 'AllRule handed back an object unexpectedly')

    def test_invalid_net_missing_comma(self):
        self._check_invalid_rawrule('all')  # missing comma

    def test_invalid_net_non_AllRule(self):
        self._check_invalid_rawrule('dbus,')  # not a all rule

    # no localvars, therefore we can't break anything inside the class variables
    # def test_empty_all_data_1(self):


class WriteAllTestAATest(AATest):
    tests = (
        #  raw rule                                     clean rule
        ('     all         ,    # foo        ',         'all, # foo'),
        ('    audit     all ,',                         'audit all,'),
        ('   deny all             ,# foo bar',          'deny all, # foo bar'),
        ('   allow all        ,# foo bar',              'allow all, # foo bar'),
        ('   allow all   ,',                            'allow all,'),
        ('   priority =  -2 allow all   ,',             'priority=-2 allow all,'),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(AllRule.match(rawrule))
        obj = AllRule.create_instance(rawrule)
        clean = obj.get_clean()
        raw = obj.get_raw()

        self.assertEqual(expected.strip(), clean, 'unexpected clean rule')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')

    def test_write_manually(self):
        obj = AllRule(allow_keyword=True)

        expected = '    allow all,'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')


class AllCoveredTest(AATest):
    def _run_test(self, param, expected):
        obj = AllRule.create_instance(self.rule)
        check_obj = AllRule.create_instance(param)

        self.assertTrue(AllRule.match(param))

        self.assertEqual(obj.is_equal(check_obj), expected[0], 'Mismatch in is_equal, expected {}'.format(expected[0]))
        self.assertEqual(obj.is_equal(check_obj, True), expected[1], 'Mismatch in is_equal/strict, expected {}'.format(expected[1]))

        self.assertEqual(obj.is_covered(check_obj), expected[2], 'Mismatch in is_covered, expected {}'.format(expected[2]))
        self.assertEqual(obj.is_covered(check_obj, True, True), expected[3], 'Mismatch in is_covered/exact, expected {}'.format(expected[3]))


class AllCoveredTest_01(AllCoveredTest):
    rule = 'all,'

    tests = (
        #   rule                 equal  strict equal    covered     covered exact
        ('           all,',     (True,  True,           True,       True)),
        ('     allow all,',     (True,  False,          True,       True)),
        ('audit      all,',     (False, False,          False,      False)),
        ('audit deny all,',     (False, False,          False,      False)),
        ('      deny all,',     (False, False,          False,      False)),
    )


class AllCoveredTest_02(AllCoveredTest):
    rule = 'audit all,'

    tests = (
        #   rule                 equal  strict equal covered  covered exact
        ('      all,',          (False, False,       True,    False)),
        ('audit all,',          (True,  True,        True,    True)),
    )


class AllCoveredTest_03(AllCoveredTest):
    rule = 'deny all,'

    tests = (
        #   rule                 equal  strict equal  covered  covered exact
        ('      deny all,',     (True,  True,         True,    True)),
        ('audit deny all,',     (False, False,        False,   False)),
        ('           all,',     (False, False,        False,   False)),  # XXX should covered be true here?
    )


class AllCoveredTest_Invalid(AATest):
    def test_invalid_is_covered(self):
        raw_rule = 'all,'

        class SomeOtherClass(AllRule):
            pass

        obj = AllRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_invalid_is_equal(self):
        raw_rule = 'all,'

        class SomeOtherClass(AllRule):
            pass

        obj = AllRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_equal(testobj)


class AllSeverityTest(AATest):
    tests = (
        ('all,', 10),
    )

    def _run_test(self, params, expected):
        sev_db = severity.Severity('../severity.db', 'unknown')
        obj = AllRule.create_instance(params)
        rank = obj.severity(sev_db)
        self.assertEqual(rank, expected)


class AllLogprofHeaderTest(AATest):
    tests = (
        ('all,',                       [                                'All', _('Allow everything')]),  # noqa: E201
        ('deny all,',                  [_('Qualifier'), 'deny',         'All', _('Allow everything')]),
        ('allow all,',                 [_('Qualifier'), 'allow',        'All', _('Allow everything')]),
        ('audit deny all,',            [_('Qualifier'), 'audit deny',   'All', _('Allow everything')]),
    )

    def _run_test(self, params, expected):
        obj = AllRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)


# --- tests for AllRuleset --- #

class AllRulesTest(AATest):
    def test_empty_ruleset(self):
        ruleset = AllRuleset()
        ruleset_2 = AllRuleset()
        self.assertEqual([], ruleset.get_raw(2))
        self.assertEqual([], ruleset.get_clean(2))
        self.assertEqual([], ruleset_2.get_raw(2))
        self.assertEqual([], ruleset_2.get_clean(2))

    def test_ruleset_1(self):
        ruleset = AllRuleset()
        rules = (
            'all,',
            'all,',
        )

        expected_raw = [
            'all,',
            'all,',
            '',
        ]

        expected_clean = [
            'all,',
            'all,',
            '',
        ]

        for rule in rules:
            ruleset.add(AllRule.create_instance(rule))

        self.assertEqual(expected_raw, ruleset.get_raw())
        self.assertEqual(expected_clean, ruleset.get_clean())

    def test_ruleset_2(self):
        ruleset = AllRuleset()
        rules = (
            'all,',
            'allow all,',
            'deny all, # example comment',
        )

        expected_raw = [
            '  all,',
            '  allow all,',
            '  deny all, # example comment',
            '',
        ]

        expected_clean = [
            '  deny all, # example comment',
            '',
            '  all,',
            '  allow all,',
            '',
        ]

        for rule in rules:
            ruleset.add(AllRule.create_instance(rule))

        self.assertEqual(expected_raw, ruleset.get_raw(1))
        self.assertEqual(expected_clean, ruleset.get_clean(1))


class AllGlobTestAATest(AATest):
    def setUp(self):
        self.ruleset = AllRuleset()

    def test_glob(self):
        with self.assertRaises(NotImplementedError):
            # get_glob is not available for all rules
            self.ruleset.get_glob('all,')

    def test_glob_ext(self):
        with self.assertRaises(NotImplementedError):
            # get_glob_ext is not available for all rules
            self.ruleset.get_glob_ext('all,')


class AllDeleteTestAATest(AATest):
    pass


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#! /usr/bin/python3
# ------------------------------------------------------------------
#
#    Copyright (C) 2020 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

import unittest

import apparmor.aa as aa
from common_test import AATest, setup_aa, setup_all_loops, skip_active_profiles, skip_extra_profiles


class TestFoo(AATest):
    # Make sure the python code can parse all profiles shipped with AppArmor.
    # If this fails, read_profiles() / read_inactive_profiles() will raise an exception.
    #
    # Checking for the number of read profiles is mostly done to ensure *something* is read
    # (to make sure an empty or non-existing directory won't make this test useless).

    def test_active_profiles(self):
        aa.read_profiles(skip_profiles=skip_active_profiles)

        self.assertGreaterEqual(len(aa.active_profiles.profile_names), 42)

    def test_extra_profiles(self):
        aa.read_inactive_profiles(skip_profiles=skip_extra_profiles)

        self.assertGreaterEqual(len(aa.extra_profiles.profile_names), 100)


setup_aa(aa)
setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#! /usr/bin/python3
# ------------------------------------------------------------------
#
#    Copyright (C) 2013 Kshitij Gupta <kgupta8592@gmail.com>
#    Copyright (C) 2015 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

import re
import unittest
from copy import deepcopy

from apparmor.common import AppArmorBug, AppArmorException, convert_regexp
from apparmor.aare import AARE, convert_expression_to_aare
from common_test import AATest, setup_all_loops


class TestConvert_regexp(AATest):
    tests = (
        ('/foo',        '^/foo$'),
        ('/{foo,bar}',  '^/(foo|bar)$'),
        # ('/\{foo,bar}', '^/\{foo,bar}$'),  # XXX gets converted to ^/\(foo|bar)$
        ('/fo[abc]',    '^/fo[abc]$'),
        ('/foo bar',    '^/foo bar$'),
        (r'/x\y',       r'^/x\y$'),
        (r'/x\[y',      r'^/x\[y$'),
        ('/x\\y',       '^/x\\y$'),
        ('/fo?',        '^/fo[^/\000]$'),
        ('/foo/*',      '^/foo/(((?<=/)[^/\000]+)|((?<!/)[^/\000]*))$'),
        ('/foo/**.bar', '^/foo/(((?<=/)[^\000]+)|((?<!/)[^\000]*))\\.bar$'),
    )

    def _run_test(self, params, expected):
        self.assertEqual(convert_regexp(params), expected)


class Test_convert_expression_to_aare(AATest):
    tests = (
        # note that \ always needs to be escaped in python, so \\ is actually just \ in the string
        ('/foo',      '/foo'),
        ('/foo?',     '/foo\\?'),
        ('/foo*',     '/foo\\*'),
        (r'/foo\*',   r'/foo\\\*'),  # raw string, no backslash doubling
        ('/foo[bar]', '/foo\\[bar\\]'),
        ('/foo{bar}', '/foo\\{bar\\}'),
        ('/foo{',     '/foo\\{'),
        ('/foo\\',    '/foo\\\\'),
        ('/foo"',     '/foo\\"'),
        ('}]"\\[{',   '\\}\\]\\"\\\\\\[\\{'),
    )

    def _run_test(self, params, expected):
        self.assertEqual(convert_expression_to_aare(params), expected)


class TestConvert_regexpAndAAREMatch(AATest):
    tests = (
        #  aare                path to check                   match expected?
        (('/foo/**/bar/',     '/foo/user/tools/bar/'),         True),
        (('/foo/**/bar/',     '/foo/apparmor/bar/'),           True),
        (('/foo/**/bar/',     '/foo/apparmor/bar'),            False),
        (('/foo/**/bar/',     '/a/foo/apparmor/bar/'),         False),
        (('/foo/**/bar/',     '/foo/apparmor/bar/baz'),        False),

        (('/foo/*/bar/',      '/foo/apparmor/bar/'),           True),
        (('/foo/*/bar/',      '/foo/apparmor/tools/bar/'),     False),
        (('/foo/*/bar/',      '/foo/apparmor/bar'),            False),

        (('/foo/user/ba?/',   '/foo/user/bar/'),               True),
        (('/foo/user/ba?/',   '/foo/user/bar/apparmor/'),      False),
        (('/foo/user/ba?/',   '/foo/user/ba/'),                False),
        (('/foo/user/ba?/',   '/foo/user/ba//'),               False),

        (('/foo/user/bar/**', '/foo/user/bar/apparmor'),       True),
        (('/foo/user/bar/**', '/foo/user/bar/apparmor/tools'), True),
        (('/foo/user/bar/**', '/foo/user/bar/'),               False),

        (('/foo/user/bar/*',  '/foo/user/bar/apparmor'),       True),
        (('/foo/user/bar/*',  '/foo/user/bar/apparmor/tools'), False),
        (('/foo/user/bar/*',  '/foo/user/bar/'),               False),
        (('/foo/user/bar/*',  '/foo/user/bar/apparmor/'),      False),

        (('/foo/**.jpg',      '/foo/bar/baz/foobar.jpg'),      True),
        (('/foo/**.jpg',      '/foo/bar/foobar.jpg'),          True),
        (('/foo/**.jpg',      '/foo/bar/*.jpg'),               True),
        (('/foo/**.jpg',      '/foo/bar.jpg'),                 True),
        (('/foo/**.jpg',      '/foo/**.jpg'),                  True),
        (('/foo/**.jpg',      '/foo/*.jpg'),                   True),
        (('/foo/**.jpg',      '/foo/barjpg'),                  False),
        (('/foo/**.jpg',      '/foo/.*'),                      False),
        (('/foo/**.jpg',      '/bar.jpg'),                     False),
        (('/foo/**.jpg',      '/**.jpg'),                      False),
        (('/foo/**.jpg',      '/*.jpg'),                       False),
        (('/foo/**.jpg',      '/foo/*.bar'),                   False),

        (('/foo/{**,}',       '/foo/'),                        True),
        (('/foo/{**,}',       '/foo/bar'),                     True),
        (('/foo/{**,}',       '/foo/bar/'),                    True),
        (('/foo/{**,}',       '/foo/bar/baz'),                 True),
        (('/foo/{**,}',       '/foo/bar/baz/'),                True),
        (('/foo/{**,}',       '/bar/'),                        False),

        (('/foo/{,**}',       '/foo/'),                        True),
        (('/foo/{,**}',       '/foo/bar'),                     True),
        (('/foo/{,**}',       '/foo/bar/'),                    True),
        (('/foo/{,**}',       '/foo/bar/baz'),                 True),
        (('/foo/{,**}',       '/foo/bar/baz/'),                True),
        (('/foo/{,**}',       '/bar/'),                        False),

        (('/foo/a[bcd]e',     '/foo/abe'),                     True),
        (('/foo/a[bcd]e',     '/foo/abend'),                   False),
        (('/foo/a[bcd]e',     '/foo/axe'),                     False),

        (('/foo/a[b-d]e',     '/foo/abe'),                     True),
        (('/foo/a[b-d]e',     '/foo/ace'),                     True),
        (('/foo/a[b-d]e',     '/foo/abend'),                   False),
        (('/foo/a[b-d]e',     '/foo/axe'),                     False),

        (('/foo/a[^bcd]e',    '/foo/abe'),                     False),
        (('/foo/a[^bcd]e',    '/foo/abend'),                   False),
        (('/foo/a[^bcd]e',    '/foo/axe'),                     True),

        (('/foo/{foo,bar,user,other}/bar/',                       '/foo/user/bar/'),               True),
        (('/foo/{foo,bar,user,other}/bar/',                       '/foo/bar/bar/'),                True),
        (('/foo/{foo,bar,user,other}/bar/',                       '/foo/wrong/bar/'),              False),

        (('/foo/{foo,bar,user,other}/test,ca}se/{aa,sd,nd}/bar/', '/foo/user/test,ca}se/aa/bar/'), True),
        (('/foo/{foo,bar,user,other}/test,ca}se/{aa,sd,nd}/bar/', '/foo/bar/test,ca}se/sd/bar/'),  True),
        (('/foo/{foo,bar,user,other}/test,ca}se/{aa,sd,nd}/bar/', '/foo/wrong/user/bar/'),         False),
        (('/foo/{foo,bar,user,other}/test,ca}se/{aa,sd,nd}/bar/', '/foo/user/wrong/bar/'),         False),
        (('/foo/{foo,bar,user,other}/test,ca}se/{aa,sd,nd}/bar/', '/foo/wrong/aa/bar/'),           False),
    )

    def _run_test(self, params, expected):
        regex, path = params
        parsed_regex = re.compile(convert_regexp(regex))
        self.assertEqual(bool(parsed_regex.search(path)), expected, 'Incorrectly Parsed regex: ' + regex)

        aare_obj = AARE(regex, True)
        self.assertEqual(aare_obj.match(path), expected, 'Incorrectly parsed AARE object: ' + regex)
        if not ('*' in path or '{' in path or '}' in path or '?' in path):
            self.assertEqual(aare_obj.match(AARE(path, False)), expected, 'Incorrectly parsed AARE object: AARE({})'.format(regex))

    def test_multi_usage(self):
        aare_obj = AARE('/foo/*', True)
        self.assertTrue(aare_obj.match('/foo/bar'))
        self.assertFalse(aare_obj.match('/foo/bar/'))
        self.assertTrue(aare_obj.match('/foo/asdf'))

    def test_match_against_AARE_1(self):
        aare_obj_1 = AARE('@{foo}/[a-d]**', True)
        aare_obj_2 = AARE('@{foo}/[a-d]**', True)
        self.assertTrue(aare_obj_1.match(aare_obj_2))
        self.assertTrue(aare_obj_1.is_equal(aare_obj_2))

    def test_match_against_AARE_2(self):
        aare_obj_1 = AARE('@{foo}/[a-d]**', True)
        aare_obj_2 = AARE('@{foo}/*[a-d]*', True)
        self.assertFalse(aare_obj_1.match(aare_obj_2))
        self.assertFalse(aare_obj_1.is_equal(aare_obj_2))

    def test_match_invalid_1(self):
        aare_obj = AARE('@{foo}/[a-d]**', True)
        with self.assertRaises(AppArmorBug):
            aare_obj.match(set())


class TestAAREMatchFromLog(AATest):
    tests = (
        #  AARE             log event     match expected?
        (('/foo/bar',       '/foo/bar'),  True),
        (('/foo/*',         '/foo/bar'),  True),
        (('/**',            '/foo/bar'),  True),
        (('/foo/*',         '/bar/foo'),  False),
        (('/foo/*',         '/foo/"*'),   True),
        (('/foo/bar',       '/foo/*'),    False),
        (('/foo/?',         '/foo/('),    True),
        (('/foo/{bar,baz}', '/foo/bar'),  True),
        (('/foo/{bar,baz}', '/foo/bars'), False),
    )

    def _run_test(self, params, expected):
        regex, log_event = params
        aare_obj_1 = AARE(regex, True)
        aare_obj_2 = AARE(log_event, True, log_event=True)
        self.assertEqual(aare_obj_1.match(aare_obj_2), expected)


class TestAAREIsEqual(AATest):
    tests = (
        # regex     is path?  check for  expected
        (('/foo',   True,     '/foo'),   True),
        (('@{foo}', True,     '@{foo}'), True),
        (('/**',    True,     '/foo'),   False),
    )

    def _run_test(self, params, expected):
        regex, is_path, check_for = params
        aare_obj_1 = AARE(regex, is_path)
        aare_obj_2 = AARE(check_for, is_path)
        self.assertEqual(expected, aare_obj_1.is_equal(check_for))
        self.assertEqual(expected, aare_obj_1.is_equal(aare_obj_2))

    def test_is_equal_invalid_1(self):
        aare_obj = AARE('/foo/**', True)
        with self.assertRaises(AppArmorBug):
            aare_obj.is_equal(42)


class TestAAREIs_eq(AATest):
    tests = (
        # regex     is path?  check for  expected
        (('/foo',   True,     '/foo'),   True),
        (('@{foo}', True,     '@{foo}'), True),
        (('/**',    True,     '/foo'),   False),
    )

    def _run_test(self, params, expected):
        regex, is_path, check_for = params
        aare_obj_1 = AARE(regex, is_path)
        aare_obj_2 = AARE(check_for, is_path)

        self.assertEqual(aare_obj_1 == aare_obj_2, expected)
        self.assertFalse(aare_obj_1 == check_for)

    def test_is_eq_invalid_1(self):
        aare_obj = AARE('/foo/**', True)
        self.assertFalse(aare_obj == 42)


class TestAAREIsPath(AATest):
    tests = (
        # regex       is path?  match for   expected
        (('/foo*',    True,     '/foobar'), True),
        (('@{PROC}/', True,     '/foobar'), False),
        (('foo*',     False,    'foobar'),  True),
        (('{/a,/b}',  True,     '/a'),      True),
        (('{@{X},/b}', True,    '/b'),      True),
    )

    def _run_test(self, params, expected):
        regex, is_path, check_for = params
        aare_obj = AARE(regex, is_path)
        self.assertEqual(expected, aare_obj.match(check_for))

    def test_path_missing_slash(self):
        with self.assertRaises(AppArmorException):
            AARE('foo*', True)

    def test_path_bad_alternative(self):
        with self.assertRaises(AppArmorException):
            AARE('{/a,@{foo},invalid}', True)

    def test_path_bad_alternative2(self):
        with self.assertRaises(AppArmorException):
            AARE('{invalid,@{X},/X}', True)

    def test_path_bad_alternative3(self):
        with self.assertRaises(AppArmorException):
            AARE('{/foo,@{invalid,/X}', True)


class TestAARERepr(AATest):
    def test_repr(self):
        obj = AARE('/foo', True)
        self.assertEqual(str(obj), "AARE('/foo')")


class TestAAREDeepcopy(AATest):
    tests = (
        # regex       is path?  log event  expected (dummy value)
        (AARE('/foo', False),              True),
        (AARE('/foo', False,    True),     True),
        (AARE('/foo', True),               True),
        (AARE('/foo', True,     True),     True),
    )

    def _run_test(self, params, expected):
        dup = deepcopy(params)

        self.assertTrue(params.match('/foo'))
        self.assertTrue(dup.match('/foo'))

        self.assertEqual(params.regex, dup.regex)
        self.assertEqual(params.orig_regex, dup.orig_regex)
        self.assertEqual(params.orig_regex, dup.orig_regex)


class TestAAREglobPath(AATest):
    tests = (
        # _run_test() will also run each test with '/' appended
        # regex                  expected AARE.regex
        ('/foo/bar/baz**',       '/foo/bar/**'),
        ('/foo/bar/**baz',       '/foo/bar/**'),
        ('/foo/bar/fo**baz',     '/foo/bar/**'),
        ('/foo/bar/**foo**',     '/foo/bar/**'),
        ('/foo/bar/**f?o**',     '/foo/bar/**'),
        ('/foo/bar/**fo[a-z]**', '/foo/bar/**'),

        ('/foo/bar/baz',         '/foo/bar/*'),
        ('/foo/bar/baz*',        '/foo/bar/*'),
        ('/foo/bar/*baz',        '/foo/bar/*'),
        ('/foo/bar/fo*baz',      '/foo/bar/*'),
        ('/foo/bar/*foo*',       '/foo/bar/*'),

        ('/foo/bar/b[a-z]z',     '/foo/bar/*'),
        ('/foo/bar/{bar,baz}',   '/foo/bar/*'),
        ('/foo/bar/{bar,ba/z}',  '/foo/bar/{bar,ba/*'),  # XXX
        ('/foo/*/baz',           '/foo/*/*'),

        ('/foo/bar/**',          '/foo/**'),
        ('/foo/bar/*',           '/foo/**'),

        ('/foo/**/*',            '/foo/**'),
        ('/foo/*/**',            '/foo/**'),
        ('/foo/*/*',             '/foo/**'),

        ('/foo',                 '/*'),
        ('/b*',                  '/*'),
        ('/*b',                  '/*'),
        ('/*',                   '/*'),
        ('/*.foo',               '/*'),
        ('/**.foo',              '/**'),
        ('/foo/*',               '/**'),
        ('/usr/foo/*',           '/usr/**'),
        ('/usr/foo/**',          '/usr/**'),
        ('/usr/foo/bar**',       '/usr/foo/**'),
        ('/usr/foo/**bar',       '/usr/foo/**'),
        ('/usr/bin/foo**bar',    '/usr/bin/**'),
        ('/usr/foo/**/bar',      '/usr/foo/**/*'),
        ('/usr/foo/**/*',        '/usr/foo/**'),
        ('/usr/foo/*/bar',       '/usr/foo/*/*'),
        ('/usr/bin/foo*bar',     '/usr/bin/*'),
        ('/usr/bin/*foo*',       '/usr/bin/*'),
        ('/usr/foo/*/*',         '/usr/foo/**'),
        ('/usr/foo/*/**',        '/usr/foo/**'),
        ('/**',                  '/**'),
    )

    def _run_test(self, params, expected):
        # test for files
        oldpath = AARE(params, True)
        newpath = oldpath.glob_path()
        self.assertEqual(expected, newpath.regex)

        # test for directories
        oldpath = AARE(params + '/', True)
        newpath = oldpath.glob_path()
        self.assertEqual(expected + '/', newpath.regex)


class TestAAREglobPathWithExt(AATest):
    tests = (
        # _run_test() will also run each test with '/' appended
        # regex                     expected AARE.regex

        # no extension - shouldn't change
        ('/foo/bar/baz**',       '/foo/bar/baz**'),
        ('/foo/bar/**baz',       '/foo/bar/**baz'),
        ('/foo/bar/fo**baz',     '/foo/bar/fo**baz'),
        ('/foo/bar/**foo**',     '/foo/bar/**foo**'),
        ('/foo/bar/**f?o**',     '/foo/bar/**f?o**'),
        ('/foo/bar/**fo[a-z]**', '/foo/bar/**fo[a-z]**'),

        ('/foo/bar/baz',         '/foo/bar/baz'),
        ('/foo/bar/baz*',        '/foo/bar/baz*'),
        ('/foo/bar/*baz',        '/foo/bar/*baz'),
        ('/foo/bar/fo*baz',      '/foo/bar/fo*baz'),
        ('/foo/bar/*foo*',       '/foo/bar/*foo*'),

        ('/foo/bar/b[a-z]z',     '/foo/bar/b[a-z]z'),
        ('/foo/bar/{bar,baz}',   '/foo/bar/{bar,baz}'),
        ('/foo/bar/{bar,ba/z}',  '/foo/bar/{bar,ba/z}'),
        ('/foo/*/baz',           '/foo/*/baz'),

        ('/foo/bar/**',          '/foo/bar/**'),
        ('/foo/bar/*',           '/foo/bar/*'),

        ('/foo/**/*',            '/foo/**/*'),
        ('/foo/*/**',            '/foo/*/**'),
        ('/foo/*/*',             '/foo/*/*'),

        ('/*',                   '/*'),
        ('/**',                  '/**'),

        ('/foo/bar',             '/foo/bar'),
        ('/foo/**/bar',          '/foo/**/bar'),
        ('/foo.bar',             '/*.bar'),
        ('/*.foo',               '/*.foo'),
        ('/usr/*.bar',           '/**.bar'),
        ('/usr/**.bar',          '/**.bar'),
        ('/usr/foo**.bar',       '/usr/**.bar'),
        ('/usr/foo*.bar',        '/usr/*.bar'),
        ('/usr/fo*oo.bar',       '/usr/*.bar'),
        ('/usr/*foo*.bar',       '/usr/*.bar'),
        ('/usr/**foo.bar',       '/usr/**.bar'),
        ('/usr/*foo.bar',        '/usr/*.bar'),
        ('/usr/foo.b*',          '/usr/*.b*'),

        # with extension added
        ('/foo/bar/baz**.xy',       '/foo/bar/**.xy'),
        ('/foo/bar/**baz.xy',       '/foo/bar/**.xy'),
        ('/foo/bar/fo**baz.xy',     '/foo/bar/**.xy'),
        ('/foo/bar/**foo**.xy',     '/foo/bar/**.xy'),
        ('/foo/bar/**f?o**.xy',     '/foo/bar/**.xy'),
        ('/foo/bar/**fo[a-z]**.xy', '/foo/bar/**.xy'),

        ('/foo/bar/baz.xy',         '/foo/bar/*.xy'),
        ('/foo/bar/baz*.xy',        '/foo/bar/*.xy'),
        ('/foo/bar/*baz.xy',        '/foo/bar/*.xy'),
        ('/foo/bar/fo*baz.xy',      '/foo/bar/*.xy'),
        ('/foo/bar/*foo*.xy',       '/foo/bar/*.xy'),

        ('/foo/bar/b[a-z]z.xy',     '/foo/bar/*.xy'),
        ('/foo/bar/{bar,baz}.xy',   '/foo/bar/*.xy'),
        ('/foo/bar/{bar,ba/z}.xy',  '/foo/bar/{bar,ba/*.xy'),  # XXX
        ('/foo/*/baz.xy',           '/foo/*/*.xy'),

        ('/foo/bar/**.xy',          '/foo/**.xy'),
        ('/foo/bar/*.xy',           '/foo/**.xy'),

        ('/foo/**/*.xy',            '/foo/**.xy'),
        ('/foo/*/**.xy',            '/foo/**.xy'),
        ('/foo/*/*.xy',             '/foo/**.xy'),

        ('/*.foo',                  '/*.foo'),
        ('/**.foo',                 '/**.foo'),

        ('/foo.baz',                '/*.baz'),
        ('/b*.baz',                 '/*.baz'),
        ('/*b.baz',                 '/*.baz'),
        ('/foo/*.baz',              '/**.baz'),
        ('/usr/foo/*.baz',          '/usr/**.baz'),
        ('/usr/foo/**.baz',         '/usr/**.baz'),
        ('/usr/foo/bar**.baz',      '/usr/foo/**.baz'),
        ('/usr/foo/**bar.baz',      '/usr/foo/**.baz'),
        ('/usr/bin/foo**bar.baz',   '/usr/bin/**.baz'),
        ('/usr/foo/**/bar.baz',     '/usr/foo/**/*.baz'),
        ('/usr/foo/**/*.baz',       '/usr/foo/**.baz'),
        ('/usr/foo/*/bar.baz',      '/usr/foo/*/*.baz'),
        ('/usr/bin/foo*bar.baz',    '/usr/bin/*.baz'),
        ('/usr/bin/*foo*.baz',      '/usr/bin/*.baz'),
        ('/usr/foo/*/*.baz',        '/usr/foo/**.baz'),
        ('/usr/foo/*/**.baz',       '/usr/foo/**.baz'),
    )

    def _run_test(self, params, expected):
        # test for files
        oldpath = AARE(params, True)
        newpath = oldpath.glob_path_withext()
        self.assertEqual(expected, newpath.regex)

        # test for directories - should be kept unchanged
        oldpath = AARE(params + '/', True)
        newpath = oldpath.glob_path_withext()
        self.assertEqual(params + '/', newpath.regex)  # note that we compare to params, not expected here


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#!/usr/bin/python3
#
#   Copyright (c) 2013 Canonical, Ltd. (All rights reserved)
#   Copyright (c) 2021 Christian Boltz
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of version 2 of the GNU General Public
#   License published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, contact Canonical Ltd.
#

from testlib import write_file


def get_rule(quantifier, perms, session, name, path, interface, member, peer):

    result = ' '

    for part in (quantifier, 'dbus', perms, session, name, path, interface, member, peer):
        if part:
            result += ' ' + part

    result += ',\n'

    return result


def gen_file(test, xres, quantifier, perms, session, name, path, interface, member, peer):
    global count

    content = ''
    content += '#\n'
    content += '#=DESCRIPTION {}\n'.format(test)
    content += '#=EXRESULT {}\n'.format(xres)
    content += '#\n'
    content += '/usr/bin/foo {\n'
    content += get_rule(quantifier, perms, session, name, path, interface, member, peer)
    content += '}\n'

    write_file('simple_tests/generated_dbus', '{}-{}.sd'.format(test, count), content)

    count += 1


def gen_files(test, xres, quantifiers, perms, sessions, names, paths, interfaces, members, peers):
    for quantifier in quantifiers:
        for perm in perms:
            for session in sessions:
                for name in names:
                    for path in paths:
                        for interface in interfaces:
                            for member in members:
                                for peer in peers:
                                    gen_file(test, xres, quantifier, perm, session, name, path, interface, member, peer)


count = 0

quantifier = ('', 'deny', 'audit')
session = ('', 'bus=session', 'bus=system', 'bus=accessibility')
path = ['', 'path=/foo/bar', 'path="/foo/bar"']
interface = ['', 'interface=com.baz', 'interface="com.baz"']
member = ['', 'member=bar', 'member="bar"']

name = ['', 'name=com.foo', 'name="com.foo"']
peer = [
    'peer=()',
    'peer=(name=com.foo)',
    'peer=(name="com.foo")',
    'peer=(label=/usr/bin/app)',
    'peer=(label="/usr/bin/app")',
    'peer=(name=com.foo label=/usr/bin/app)',
    'peer=(name="com.foo" label="/usr/bin/app")',
]

# msg_perms are the permissions that are related to sending and receiving
# messages.
msg_perms = [
    '',
    'r',
    'w',
    'rw',
    'read',
    'receive',
    'write',
    'send',
    '(r)',
    '(w)',
    '(rw)',
    '(read)',
    '(receive)',
    '(write)',
    '(send)',
    '(write, read)',
    '(send receive)',
    '(send read)',
    '(receive write)',
]

empty_tup = ('',)

gen_files('message-rules', 'PASS', quantifier, msg_perms, session,
          empty_tup, path, interface, member, peer)
gen_files('service-rules', 'PASS', quantifier, ['bind'], session,
          name, empty_tup, empty_tup, empty_tup, empty_tup)
gen_files('eavesdrop-rules', 'PASS', quantifier, ['eavesdrop'], session,
          empty_tup, empty_tup, empty_tup, empty_tup, empty_tup)
gen_file('sloppy-formatting', 'PASS', '', '(send , receive )', 'bus=session',
         '', 'path ="/foo/bar"', 'interface = com.foo', '  member=bar',
         'peer =(   label= /usr/bin/app name  ="com.foo")')
gen_file('sloppy-formatting', 'PASS', '', 'bind', 'bus =session',
         'name= com.foo', '', '', '', '')
gen_file('sloppy-formatting', 'PASS', '', 'eavesdrop', 'bus = system',
         '', '', '', '', '')

# Don't use the empty element from each array since all empty conditionals would PASS but we want all FAILs
msg_perms.remove('')
name.remove('')
path.remove('')
interface.remove('')
member.remove('')
peer.remove('peer=()')

gen_files('message-incompat', 'FAIL', quantifier, msg_perms, session, name, empty_tup, empty_tup, empty_tup, empty_tup)
gen_files('service-incompat', 'FAIL', quantifier, ('bind',), session, name, path, empty_tup, empty_tup, empty_tup)
gen_files('service-incompat', 'FAIL', quantifier, ('bind',), session, name, empty_tup, interface, empty_tup, empty_tup)
gen_files('service-incompat', 'FAIL', quantifier, ('bind',), session, name, empty_tup, empty_tup, member, empty_tup)
gen_files('service-incompat', 'FAIL', quantifier, ('bind',), session, name, empty_tup, empty_tup, empty_tup, peer)
gen_files('eavesdrop-incompat', 'FAIL', quantifier, ('eavesdrop',), session, name, path, interface, member, peer)

gen_files('pairing-unsupported', 'FAIL', quantifier, ('send', 'bind'),
          session, ('name=sn', 'label=sl'), empty_tup, empty_tup, empty_tup,
          ('peer=(name=pn)', 'peer=(label=pl)'))

# missing bus= prefix
gen_file('bad-formatting', 'FAIL', '', 'send', 'session', '', '', '', '', '')
# incorrectly formatted permissions
gen_files('bad-perms', 'FAIL', empty_tup, ('send receive', '(send', 'send)'),
          ('bus=session',), empty_tup, empty_tup, empty_tup, empty_tup, empty_tup)
# invalid permissions
gen_files('bad-perms', 'FAIL', empty_tup,
          ('a', 'x', 'Ux', 'ix', 'm', 'k', 'l', '(a)', '(x)'), empty_tup, empty_tup,
          empty_tup, empty_tup, empty_tup, empty_tup)

gen_file('duplicated-conditionals', 'FAIL', '', 'bus=1 bus=2', '', '', '', '', '', '')
gen_file('duplicated-conditionals', 'FAIL', '', 'name=1 name=2', '', '', '', '', '', '')
gen_file('duplicated-conditionals', 'FAIL', '', 'path=1 path=2', '', '', '', '', '', '')
gen_file('duplicated-conditionals', 'FAIL', '', 'interface=1 interface=2', '', '', '', '', '', '')
gen_file('duplicated-conditionals', 'FAIL', '', 'member=1 member=2', '', '', '', '', '', '')
gen_file('duplicated-conditionals', 'FAIL', '', 'peer=(name=1) peer=(name=2)', '', '', '', '', '', '')
gen_file('duplicated-conditionals', 'FAIL', '', 'peer=(label=1) peer=(label=2)', '', '', '', '', '', '')
gen_file('duplicated-conditionals', 'FAIL', '', 'peer=(name=1) peer=(label=2)', '', '', '', '', '', '')

print('Generated {} dbus tests'.format(count))



#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2020 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from collections import namedtuple
from common_test import AATest, setup_all_loops

from apparmor.common import AppArmorBug, AppArmorException
from apparmor.rule.variable import VariableRule, VariableRuleset, separate_vars
from apparmor.translations import init_translation

_ = init_translation()

exp = namedtuple('exp', ('comment', 'varname', 'mode', 'values'))

# --- tests for single VariableRule --- #


class VariableTest(AATest):
    def _compare_obj(self, obj, expected):
        # variables don't support the allow, audit, deny, or priority keyword
        self.assertEqual(False, obj.allow_keyword)
        self.assertEqual(False, obj.audit)
        self.assertEqual(False, obj.deny)
        self.assertEqual(None, obj.priority)

        self.assertEqual(expected.varname, obj.varname)
        self.assertEqual(expected.mode, obj.mode)
        self.assertEqual(expected.values, obj.values)
        self.assertEqual(expected.comment, obj.comment)


class AaTest_separate_vars(AATest):
    tests = (
        ('',                      set()),
        ('       ',               set()),
        ('  foo bar',             {'foo', 'bar'}),
        ('foo "  ',               AppArmorException),
        (' " foo ',               AppArmorException),  # half-quoted
        ('  foo bar   ',          {'foo', 'bar'}),
        ('  foo bar   # comment', {'foo', 'bar', '#', 'comment'}),  # XXX should comments be stripped?
        ('foo',                   {'foo'}),
        ('"foo" "bar baz"',       {'foo', 'bar baz'}),
        ('foo "bar baz" xy',      {'foo', 'bar baz', 'xy'}),
        ('foo "bar baz ',         AppArmorException),  # half-quoted
        ('  " foo" bar',          {' foo', 'bar'}),
        ('  " foo" bar x',        {' foo', 'bar', 'x'}),
        ('""',                    {''}),  # empty value
        ('"" foo',                {'', 'foo'}),  # empty value + 'foo'
        ('"" foo "bar"',          {'', 'foo', 'bar'}),  # empty value + 'foo' + 'bar' (bar has superfluous quotes)
        ('"bar"',                 {'bar'}),  # 'bar' with superfluous quotes
    )

    def _run_test(self, params, expected):
        if expected == AppArmorException:
            with self.assertRaises(expected):
                separate_vars(params)
        else:
            result = separate_vars(params)
            self.assertEqual(result, expected)


class VariableTestParse(VariableTest):
    tests = (
        # rawrule                                  comment      varname    mode  values
        ('@{foo}=/bar',                        exp('',           '@{foo}', '=',  {'/bar'})),
        ('@{foo}+=/bar',                       exp('',           '@{foo}', '+=', {'/bar'})),
        ('  @{foo} =   /bar   ',               exp('',           '@{foo}', '=',  {'/bar'})),
        ('  @{foo}   +=    /bar',              exp('',           '@{foo}', '+=', {'/bar'})),
        ('  @{foo} =   /bar        # comment', exp(' # comment', '@{foo}', '=',  {'/bar'})),
        ('  @{foo}   +=    /bar   # comment',  exp(' # comment', '@{foo}', '+=', {'/bar'})),
        ('@{foo}=/bar /baz',                   exp('',           '@{foo}', '=',  {'/bar', '/baz'})),
        ('@{foo} = "/bar,"   # comment',       exp(' # comment', '@{foo}', '=',  {'/bar,'})),  # value with trailing comma, needs to be quoted
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(VariableRule.match(rawrule))
        obj = VariableRule.create_instance(rawrule)
        self.assertEqual(rawrule.strip(), obj.raw_rule)
        self._compare_obj(obj, expected)


class VariableTestParseInvalid(VariableTest):
    tests = (
        # rawrule                 matches regex     exception
        ('@{foo} =',                   (False, AppArmorException)),
        ('@ {foo} =      # comment',   (False, AppArmorException)),
        ('@ {foo} =      ',            (False, AppArmorException)),
        ('@{foo} = /foo,',             (True,  AppArmorException)),  # trailing comma
        ('@{foo} = /foo,   ',          (True,  AppArmorException)),  # trailing comma
        ('@{foo} = /foo,   # comment', (True,  AppArmorException)),  # trailing comma
        ('@{foo} = /foo, /bar',        (True,  AppArmorException)),  # trailing comma in first value
        ('@{foo = /foo f',             (True,  AppArmorException)),  # variable name broken, missing }
    )

    def _run_test(self, rawrule, expected):
        self.assertEqual(VariableRule.match(rawrule), expected[0])
        with self.assertRaises(expected[1]):
            VariableRule.create_instance(rawrule)


class VariableFromInit(VariableTest):
    tests = (
        # VariableRule object                                        comment   varname    mode  values
        (VariableRule('@{foo}', '=',  {'/bar'}),                  exp('',      '@{foo}', '=',  {'/bar'})),
        (VariableRule('@{foo}', '+=', {'/bar'}),                  exp('',      '@{foo}', '+=', {'/bar'})),
        (VariableRule('@{foo}', '=',  {'/bar', '/baz'}),          exp('',      '@{foo}', '=',  {'/bar', '/baz'})),
        (VariableRule('@{foo}', '+=', {'/bar', '/baz'}),          exp('',      '@{foo}', '+=', {'/bar', '/baz'})),
        (VariableRule('@{foo}', '=',  {'/bar'}, comment='# cmt'), exp('# cmt', '@{foo}', '=',  {'/bar'})),
        (VariableRule('@{foo}', '+=', {'/bar'}, comment='# cmt'), exp('# cmt', '@{foo}', '+=', {'/bar'})),
    )

    def _run_test(self, obj, expected):
        self._compare_obj(obj, expected)


class InvalidVariableInit(AATest):
    tests = (
        # init params                expected exception
        ((None,     '=',  ['/bar']), AppArmorBug),  # varname not a str
        (('',       '=',  ['/bar']), AppArmorException),  # empty varname
        (('foo',    '=',  ['/bar']), AppArmorException),  # varname not starting with '@{'
        (('foo',    '=',  ['/bar']), AppArmorException),  # varname not starting with '@{'

        (('@{foo}', '',   ['/bar']), AppArmorBug),  # mode not '=' or '+='
        (('@{foo}', '-=', ['/bar']), AppArmorBug),  # mode not '=' or '+='
        (('@{foo}', ' ',  ['/bar']), AppArmorBug),  # mode not '=' or '+='
        (('@{foo}', None, ['/bar']), AppArmorBug),  # mode not '=' or '+='

        (('@{foo}', '=',  None),     AppArmorBug),  # values not a set
        (('@{foo}', '=',  set()),    AppArmorException),  # empty values
    )

    def _run_test(self, params, expected):
        with self.assertRaises(expected):
            VariableRule(*params)

    def test_missing_params_1(self):
        with self.assertRaises(TypeError):
            VariableRule()

    def test_missing_params_2(self):
        with self.assertRaises(TypeError):
            VariableRule('@{foo}')

    def test_missing_params_3(self):
        with self.assertRaises(TypeError):
            VariableRule('@{foo}', '=')

    def test_invalid_audit(self):
        with self.assertRaises(AppArmorBug):
            VariableRule('@{foo}', '=', '/bar', audit=True)

    def test_invalid_deny(self):
        with self.assertRaises(AppArmorBug):
            VariableRule('@{foo}', '=', '/bar', deny=True)

    def test_invalid_priority_1(self):
        with self.assertRaises(AppArmorBug):
            VariableRule('@{foo}', '=', '/bar', priority=98)

    def test_invalid_priority_2(self):
        with self.assertRaises(AppArmorBug):
            VariableRule('@{foo}', '=', '/bar', priority=0)


class InvalidVariableTest(AATest):
    def _check_invalid_rawrule(self, rawrule, matches_regex=False):
        obj = None
        self.assertEqual(VariableRule.match(rawrule), matches_regex)
        with self.assertRaises(AppArmorException):
            obj = VariableRule.create_instance(rawrule)

        self.assertIsNone(obj, 'VariableRule handed back an object unexpectedly')

    def test_invalid_missing_values(self):
        self._check_invalid_rawrule('@{foo} =  ', matches_regex=True)  # missing values

    def test_invalid_net_non_VariableRule(self):
        self._check_invalid_rawrule('dbus,')  # not a variable rule


class WriteVariableTestAATest(AATest):
    tests = (
        #  raw rule                         clean rule
        ('  @{foo}  =  /bar   ',           '@{foo} = /bar'),
        ('  @{foo}  =  /bar   # comment',  '@{foo} = /bar'),
        ('  @{foo}  =  /bar  ""',          '@{foo} = "" /bar'),
        ('  @{foo}  +=  /bar   ',          '@{foo} += /bar'),
        ('  @{foo}  +=  /bar   # comment', '@{foo} += /bar'),
        ('  @{foo}  +=  /bar       /baz',  '@{foo} += /bar /baz'),
        ('  @{foo}  +=  /bar       /baz',  '@{foo} += /bar /baz'),
        ('  @{foo}  +=  /bar      @{baz}', '@{foo} += /bar @{baz}'),
        ('  @{foo}  +=  /bar      @{baz}', '@{foo} += /bar @{baz}'),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(VariableRule.match(rawrule))
        obj = VariableRule.create_instance(rawrule)
        clean = obj.get_clean()
        raw = obj.get_raw()

        self.assertEqual(expected.strip(), clean, 'unexpected clean rule')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')

    def test_write_manually_1(self):
        obj = VariableRule('@{foo}', '=', {'/bar'})

        expected = '    @{foo} = /bar'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')

    def test_write_manually_2(self):
        obj = VariableRule('@{foo}', '=', {'/bar', ''})

        expected = '    @{foo} = "" /bar'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')


class VariableCoveredTest(AATest):
    def _run_test(self, param, expected):
        obj = VariableRule.create_instance(self.rule)
        check_obj = VariableRule.create_instance(param)

        self.assertTrue(VariableRule.match(param))

        self.assertEqual(obj.is_equal(check_obj), expected[0], 'Mismatch in is_equal, expected {}'.format(expected[0]))
        self.assertEqual(obj.is_equal(check_obj, True), expected[1], 'Mismatch in is_equal/strict, expected {}'.format(expected[1]))

        self.assertEqual(obj.is_covered(check_obj), expected[2], 'Mismatch in is_covered, expected {}'.format(expected[2]))
        self.assertEqual(obj.is_covered(check_obj, True, True), expected[3], 'Mismatch in is_covered/exact, expected {}'.format(expected[3]))


class VariableCoveredTest_01(VariableCoveredTest):
    rule = '@{foo} = /bar'

    tests = (
        #   rule                                  equal  strict equal  covered  covered exact
        ('           @{foo} = /bar',             (True,  True,         True,    True)),
        ('           @{foo} += /bar',            (False, False,        False,   False)),
        ('           @{foo} = /bar   # comment', (True,  False,        True,    True)),
        ('           @{foo} += /bar  # comment', (False, False,        False,   False)),
        ('           @{foo} = /baz /bar',        (False, False,        False,   False)),
        ('           @{foo} += /baz /bar',       (False, False,        False,   False)),
        ('           @{foo} = /baz /bar # cmt',  (False, False,        False,   False)),
        ('           @{foo} += /baz /bar # cmt', (False, False,        False,   False)),
        ('           @{bar} = /bar',             (False, False,        False,   False)),  # different variable name
    )


class VariableCoveredTest_02(VariableCoveredTest):
    rule = '@{foo} = /bar /baz'

    tests = (
        #   rule                                  equal  strict equal  covered  covered exact
        ('           @{foo} = /bar /baz',        (True,  True,         True,    True)),
        ('           @{foo} += /bar /baz',       (False, False,        False,   False)),
        ('           @{foo} = /bar /baz # cmt',  (True,  False,        True,    True)),
        ('           @{foo} += /bar /baz # cmt', (False, False,        False,   False)),
        # changed order of values
        ('           @{foo} = /baz /bar',        (True,  False,        True,    True)),
        ('           @{foo} += /baz /bar',       (False, False,        False,   False)),
        ('           @{foo} = /baz /bar # cmt',  (True,  False,        True,    True)),
        ('           @{foo} += /baz /bar # cmt', (False, False,        False,   False)),
        # only one value
        ('           @{foo} = /bar',             (False, False,        True,    True)),
        ('           @{foo} += /bar',            (False, False,        False,   False)),
        ('           @{foo} = /bar   # comment', (False, False,        True,    True)),
        ('           @{foo} += /bar  # comment', (False, False,        False,   False)),
        ('           @{bar} = /bar',             (False, False,        False,   False)),  # different variable name
    )


class VariableCoveredTest_Invalid(AATest):
    # def test_borked_obj_is_covered_1(self):
    #     obj = VariableRule.create_instance('@{foo} = /bar')
    #
    #     testobj = VariableRule('@{foo}', '=', '/bar')
    #     testobj.mode = ''
    #
    #     with self.assertRaises(AppArmorBug):
    #         obj.is_covered(testobj)

    def test_borked_obj_is_covered_2(self):
        obj = VariableRule.create_instance('@{foo} = /bar')

        testobj = VariableRule('@{foo}', '=', {'/bar'})
        testobj.values = ''

        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_invalid_is_covered_3(self):
        raw_rule = '@{foo} = /bar'

        class SomeOtherClass(VariableRule):
            pass

        obj = VariableRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_invalid_is_equal(self):
        raw_rule = '@{foo} = /bar'

        class SomeOtherClass(VariableRule):
            pass

        obj = VariableRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_equal(testobj)


class VariableLogprofHeaderTest(AATest):
    tests = (
        ('@{foo} = /bar', [_('Variable'), '@{foo} = /bar']),
    )

    def _run_test(self, params, expected):
        obj = VariableRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)

# --- tests for VariableRuleset --- #


class VariableRulesTest(AATest):
    def test_empty_ruleset(self):
        ruleset = VariableRuleset()
        ruleset_2 = VariableRuleset()
        self.assertEqual([], ruleset.get_raw(2))
        self.assertEqual([], ruleset.get_clean(2))
        self.assertEqual([], ruleset_2.get_raw(2))
        self.assertEqual([], ruleset_2.get_clean(2))
        self.assertEqual({'=': {}, '+=': {}}, ruleset_2.get_merged_variables())

    def test_ruleset_1(self):
        ruleset = VariableRuleset()
        rules = (
            '@{foo} = /bar',
            '@{baz}= /asdf',
            '@{foo}    +=   /whatever',
            '@{foo}    +=   /morestuff',
        )

        expected_raw = [
            '@{foo} = /bar',
            '@{baz}= /asdf',
            '@{foo}    +=   /whatever',
            '@{foo}    +=   /morestuff',
            '',
        ]

        expected_clean = [
            '@{baz} = /asdf',
            '@{foo} += /morestuff',
            '@{foo} += /whatever',
            '@{foo} = /bar',
            '',
        ]

        expected_clean_unsorted = [
            '@{foo} = /bar',
            '@{baz} = /asdf',
            '@{foo} += /whatever',
            '@{foo} += /morestuff',
            '',
        ]

        expected_merged = {
            '=': {
                '@{foo}': {'/bar'},
                '@{baz}': {'/asdf'},
            },
            '+=': {
                '@{foo}': {'/whatever', '/morestuff'},
            }
        }

        for rule in rules:
            ruleset.add(VariableRule.create_instance(rule))

        self.assertEqual(expected_raw, ruleset.get_raw())
        self.assertEqual(expected_clean, ruleset.get_clean())
        self.assertEqual(expected_clean_unsorted, ruleset.get_clean_unsorted())
        self.assertEqual(expected_merged, ruleset.get_merged_variables())

    def test_ruleset_overwrite(self):
        ruleset = VariableRuleset()

        ruleset.add(VariableRule.create_instance('@{foo} = /bar'))
        with self.assertRaises(AppArmorException):
            ruleset.add(VariableRule.create_instance('@{foo} = /asdf'))  # attempt to redefine @{foo}
        self.assertEqual({'=': {'@{foo}': {'/bar'}}, '+=': {}}, ruleset.get_merged_variables())


class VariableGlobTestAATest(AATest):
    def setUp(self):
        self.ruleset = VariableRuleset()

#   def test_glob_1(self):
#       with self.assertRaises(NotImplementedError):
#           self.ruleset.get_glob('@{foo} = /bar')

    def test_glob_ext(self):
        with self.assertRaises(NotImplementedError):
            # get_glob_ext is not available for change_profile rules
            self.ruleset.get_glob_ext('@{foo} = /bar')


class VariableDeleteTestAATest(AATest):
    pass


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)
